// Copyright (C) Oleg Shparber, et al. <https://zealdocs.org>
// SPDX-License-Identifier: GPL-3.0-or-later

#include "viewprovider.h"

namespace Zeal::Sidebar {

ViewProvider::ViewProvider(QObject *parent)
    : QObject(parent)
{
}

} // namespace Zeal::Sidebar
