/* Highlight on navigation to an anchor. */
@keyframes targetNavigatedAnimation {
    from { background: #fff; }
    50% { background: #ff0; }
    to { background: #fff; }
}

*:target {
    animation: targetNavigatedAnimation .5s linear;
}
